package nts.parser;

import java.util.*;

import org.antlr.runtime.Token;

import nts.interf.*;
import nts.interf.base.IExpr;
import nts.interf.base.ILabel;
import nts.interf.base.IVisitor;
import nts.interf.expr.IAccessBasic;

public class Call implements ICall, ILabel {
	
	public void accept(IVisitor v) { v.visit(this); }
	
	private Subsystem callee;
	private Token calleeTok;
	private List<IExpr> all = new LinkedList<IExpr>(); // actual parameters + return variables
	private List<IExpr> act = new LinkedList<IExpr>(); // actual parameters
	private List<IAccessBasic> ret = new LinkedList<IAccessBasic>(); // return variable
	
	private Havoc havoc;
	void havoc(Havoc aHavoc) { havoc = aHavoc; }
	
	// interface
	public Subsystem callee() { return callee; }
	public List<IExpr> actualParameters() { return act; }
	public List<IAccessBasic> returnVars() { return ret; }
	public boolean hasHavoc() { return havoc != null; }
	public Havoc havoc() { return havoc; }
	
	Token calleeToken() { return calleeTok; }
	
	public Call() {}
	
	public void setCallee(Token aT) { calleeTok = aT; }
	public void setCallee(Subsystem aS) { callee = aS; }
	public void addAct(IExpr aE) { act.add(aE); }
	public void addRet(IAccessBasic aE) { ret.add(aE); }
	public void addToAll(IExpr aE) { all.add(aE); }
	
	private static void errRetVar(Expr e) {
		System.err.println("Semantic error at "+Common.at(e.first)+". Mismatch with callee's definition. Return (primed) variable expected.");
		System.exit(1);
	}
	// separate input and output arguments of calls, infer types
	void semantic(int nIn, int nOut, VarTable aVT) {
		
		int nTot = nIn+nOut;
		// after parsing, either 'act' and 'ret' is set, or 'all' is set
		
		// make sure that correct number of arguments and return variables is provided
		if (nTot != all.size() && nTot != act.size()+ret.size()) {
			System.err.println("Semantic error at "+Common.at(calleeTok)+". Number of in/out parameters does not match with definition of '"+callee()+"'.");
			System.exit(1);
		}
		
		// separate if not separated yet
		if (ret.size() + act.size() < nTot) {
			for (int i=0; i<nIn; i++) {
				act.add(all.get(i));
			}
			for (int i=nIn; i<nTot; i++) {
				Expr aux = (Expr)all.get(i);
				if (!(aux instanceof AccessBasic)) {
					errRetVar(aux);
				}
				ret.add((AccessBasic)aux);
			}
		}
		
		// actual parameters must be expressions over unprimed variables
		SemFlags fAct = SemFlags.createActParam();
		for (IExpr e : act) {
			Expr ee = (Expr)e;
			ee.semanticChecks(aVT,fAct);
		}
		SemFlags fRet = SemFlags.createRetParam();
		for (IAccessBasic a : ret) {
			Access aa = (Access)a;
			aa.blockWriteTo = false;
			aa.semanticChecks(aVT,fRet);
			if (!aa.entry.isPrimed()) {
				errRetVar(aa);
			}
		}
		SemFlags fHav = SemFlags.createHavoc();
		if (hasHavoc())
			havoc.semanticChecks(aVT,fHav);
	}
}
