package nts.parser;

import nts.interf.base.*;

public class Type implements IType {
	
	public void accept(IVisitor v) { v.visit(this); }
	
	private EBasicType basicType;
	
	private int dimOwn, dimRef, dimTotal; // dimTotal = dimNonRel + dimRef;
	
	// artificial type for expression lists
	// 0 -- not using expression lists
	// x>0 -- expression list of size x
	private int many_expr = 0;
	
	////////////////////////
	// Interface methods ---
	public EBasicType basicType() { return basicType; }
	
	public boolean isScalar() { return many_expr == 0 && dimTotal == 0; }
	public boolean isArray() { return many_expr == 0 && dimTotal != 0; }
	
	public boolean isArrayOfBasic() { return many_expr == 0 && dimOwn != 0 && dimRef == 0; }
	public boolean isRefToArray() { return many_expr == 0 && dimOwn == 0 && dimRef != 0; }
	public boolean isArrayOfRef() { return many_expr == 0 && dimOwn != 0 && dimRef != 0; }
	
	public int dimRef() { return dimRef; }
	public int dimOwn() { return dimOwn; }
	public int dimTotal() { return dimTotal; }
	
	// --- Interface methods
	////////////////////////
	
	void setBasicType(EBasicType aType) {
		basicType = aType;
	}
	
	public Type() {
		this(0,0);
	}
	public Type(int aDimTotal, int aDimOwn) {
		dimTotal = aDimTotal;
		dimOwn = aDimOwn;
		dimRef = dimTotal - dimOwn;
	}
	// scalar type  -- for quantifier-scoped declarations (no arrays)
	public Type(EBasicType aType) {
		basicType = aType;
		dimOwn = 0; dimRef = 0; dimTotal = 0;
	}
	// pure reference
	private Type(EBasicType aType, int aDRef) {
		basicType = aType;
		dimRef = aDRef;
		dimTotal = aDRef;
		dimOwn = 0;
	}
	static Type dummyPureArrRef(EBasicType aType, int aN) {
		return new Type(aType, aN);
	}
	static Type dummyManyPureArrRef(EBasicType aType, int aN, int c) {
		Type t = new Type(aType, aN);
		t.many_expr = c;
		return t;
	}
	
	
	// type-checking methods
	boolean isInt() { 
		return isScalar() && basicType == EBasicType.INT;
	}
	boolean isReal() { 
		return isScalar() && basicType == EBasicType.REAL;
	}
	boolean isBool() { 
		return isScalar() && basicType == EBasicType.BOOL;
	}
	boolean isNum() { 
		return isScalar() && (basicType == EBasicType.INT || basicType == EBasicType.REAL);
	}
	boolean matches(Type other) {
		return
			basicType == other.basicType &&
			dimTotal == other.dimTotal &&
			dimRef == other.dimRef &&
			many_expr == other.many_expr;
	}
//	// true if one can be replaced by the other on the RHS and still make a valid assignment
//	boolean matchesForAsgn(Type other) {
//		return
//			basicType == other.basicType &&
//			dimTotal == other.dimTotal &&
//			many_expr == other.many_expr;
//	}
	
}
